@@ -52,9 +52,6 @@ EXTRA_MIXIN               object            Class to to inject into the parents
                                             of the Mixin hierarchy.
                                             (useful for composition of Django
                                             apps)
-ORGANIZATION_MODEL        saas.Organization Replace the ``Organization`` model
-                                            (useful for composition of Django
-                                            apps)
 PAGE_SIZE                 25                Maximum number of objects to return
                                             per API calls.
 PROCESSOR                :doc:`Stripe backend<backends>`
@@ -69,9 +66,6 @@ PROVIDER_SITE_CALLABLE   None               Optional function that returns
                                             fully qualified URLs.
                                             (useful for composition of Django
                                             apps)
-ROLE_RELATION            saas.Role          Replace the ``Role`` model
-                                            (useful for composition of Django
-                                            apps)
 TERMS_OF_USE             'terms-of-use'     slug for the ``Agreement`` stating
                                             ther Terms of Use of the site.
 ========================  ================= ===========
@@ -105,6 +99,8 @@ _SETTINGS = {
         'saas.Organization'),
     'PAGE_SIZE': 25,
     'PICTURE_STORAGE_CALLABLE': None,
+    'PLATFORM_NAME': getattr(settings, 'APP_NAME',
+        os.path.basename(settings.BASE_DIR)),
     'PROCESSOR': {
         'BACKEND': 'saas.backends.stripe_processor.StripeBackend',
         'CLIENT_ID': None,
@@ -121,7 +117,8 @@ _SETTINGS = {
         'WEBHOOK_SECRET': None,
     },
     'PROCESSOR_BACKEND_CALLABLE': None,
-    'ROLE_RELATION': getattr(settings, 'SAAS_ROLE_MODEL', 'saas.Role'),
+    'PRODUCT_URL_CALLABLE': None,
+    'ROLE_MODEL': getattr(settings, 'SAAS_ROLE_MODEL', 'saas.Role'),
     'ROLE_SERIALIZER': 'saas.api.serializers.RoleSerializer',
     'USER_SERIALIZER': 'saas.api.serializers_overrides.UserSerializer',
     'USER_DETAIL_SERIALIZER': 'saas.api.serializers_overrides.UserSerializer',
@@ -143,8 +140,19 @@ VERIFICATION_KEY_RE = r'[a-f0-9]{40}'
 AUTH_USER_MODEL = getattr(
     settings, 'AUTH_USER_MODEL', 'django.contrib.auth.models.User')
 
+#: overrides the implementation of `saas.models.get_broker`
+#: This function must return an `Organization` instance.
+#: It is often necessary to override the default implementation
+#: when you are dealing with a Web hosting service and each Website
+#: has its own database of users, profiles, etc.
 BROKER_CALLABLE = _SETTINGS.get('BROKER').get('GET_INSTANCE', None)
 BROKER_FEE_PERCENTAGE = _SETTINGS.get('BROKER').get('FEE_PERCENTAGE', 0)
+
+#: overrides the implementation of `saas.utils.build_absolute_uri`
+#: This function must return fully qualified URL.
+#: It is often necessary to override the default implementation
+#: when you are dealing with a Web hosting service and each Website
+#: has its own database of users, profiles, etc.
 BUILD_ABSOLUTE_URI_CALLABLE = _SETTINGS.get('BROKER').get(
     'BUILD_ABSOLUTE_URI_CALLABLE')
 BYPASS_IMPLICIT_GRANT = _SETTINGS.get('BYPASS_IMPLICIT_GRANT')
@@ -160,13 +168,24 @@ MAX_RENEWAL_ATTEMPTS = _SETTINGS.get('MAX_RENEWAL_ATTEMPTS')
 MAX_TYPEAHEAD_CANDIDATES = _SETTINGS.get('MAX_TYPEAHEAD_CANDIDATES')
 ORGANIZATION_MODEL = _SETTINGS.get('ORGANIZATION_MODEL')
 PAGE_SIZE = _SETTINGS.get('PAGE_SIZE')
+PLATFORM_NAME = _SETTINGS.get('PLATFORM_NAME')
 PROCESSOR = _SETTINGS.get('PROCESSOR')
 PROCESSOR_BACKEND_CALLABLE = _SETTINGS.get('PROCESSOR_BACKEND_CALLABLE')
 PROCESSOR_FALLBACK = PROCESSOR.get('FALLBACK', [])
 PROCESSOR_ID = PROCESSOR.get('INSTANCE_PK', 1)
 PROCESSOR_HOOK_URL = PROCESSOR.get('WEBHOOK_URL', 'stripe/postevent')
 PROCESSOR_HOOK_SECRET = PROCESSOR.get('WEBHOOK_SECRET')
-ROLE_RELATION = _SETTINGS.get('ROLE_RELATION')
+
+#: overrides the implementation of `saas.mixins.product_url`
+#: This function must return a absolute URL from a `provider`, `subscriber`,
+#: `plan` and `request` arguments, all of which except the `provider` can be
+#: `None`. For example, the default implementation returns either:
+#: /app/, or
+#: /app/{subscriber}/, or
+#: /app/{subscriber}/{plan}/.
+PRODUCT_URL_CALLABLE = _SETTINGS.get('PRODUCT_URL_CALLABLE')
+
+ROLE_MODEL = _SETTINGS.get('ROLE_MODEL')
 ROLE_SERIALIZER = _SETTINGS.get('ROLE_SERIALIZER')
 USER_SERIALIZER = _SETTINGS.get('USER_SERIALIZER')
 USER_DETAIL_SERIALIZER = _SETTINGS.get('USER_DETAIL_SERIALIZER')
@@ -176,6 +195,9 @@ TERMS_OF_USE = _SETTINGS.get('TERMS_OF_USE')
 # BE EXTRA CAREFUL! This variable is used to bypass PermissionDenied
 # exceptions. It is solely intended as a debug flexibility nob.
 BYPASS_PERMISSION_CHECK = _SETTINGS.get('BYPASS_PERMISSION_CHECK')
+
+#: A callable function which returns a `Storage` object that will be used
+#: to upload a contact picture
 PICTURE_STORAGE_CALLABLE = _SETTINGS.get('PICTURE_STORAGE_CALLABLE')
 EXTRA_FIELD = _SETTINGS.get('EXTRA_FIELD')
 
