@@ -193,7 +193,7 @@ def get_organization_model():
 def get_role_model():
     # delayed import so we can load ``OrganizationMixinBase`` in django.conf
     from . import settings #pylint:disable=import-outside-toplevel
-    return get_model_class(settings.ROLE_RELATION, 'ROLE_RELATION')
+    return get_model_class(settings.ROLE_MODEL, 'ROLE_MODEL')
 
 
 def get_role_serializer():
@@ -390,4 +390,8 @@ def build_absolute_uri(request, location='/', provider=None, with_scheme=True):
                     with_scheme=with_scheme)
         except ImportError:
             pass
-    return request.build_absolute_uri(location)
+    if request:
+        return request.build_absolute_uri(location)
+    # If we don't have a `request` object, better to return a URL path
+    # than throwing an error.
+    return location
