@@ -323,13 +323,11 @@ class CheckoutFormMixin(CardFormMixin):
             lines_price = context.get('lines_price')
             provider = self.invoicables_provider
             context.update(
-                provider.processor_backend.get_payment_context(
-                    provider,
-                    self.organization.processor_card_key,
+                provider.processor_backend.get_payment_context(# checkout
+                    self.organization,
                     amount=lines_price.amount, unit=lines_price.unit,
                     broker_fee_amount=self.invoicables_broker_fee_amount,
-                    subscriber_email=self.organization.email,
-                    subscriber_slug=self.organization.slug))
+                    provider=provider, broker=get_broker()))
         except ProcessorConnectionError:
             messages.error(self.request, _("The payment processor is "\
                 "currently unreachable. Sorry for the inconvienience."))
@@ -349,10 +347,11 @@ class CheckoutFormMixin(CardFormMixin):
         if redirect_path:
             return redirect_path
         if self.sole_provider:
-            return product_url(self.sole_provider, self.organization,
+            # XXX If there is only one plan we should add it here
+            # as an argument.
+            return product_url(subscriber=self.organization,
                 request=self.request)
-        return product_url(get_broker(), self.organization,
-            request=self.request)
+        return product_url(subscriber=self.organization, request=self.request)
 
 
 class CardUpdateView(CardFormMixin, FormView):
@@ -394,11 +393,9 @@ class CardUpdateView(CardFormMixin, FormView):
             # computed in `InvoicablesMixin.get_context_data`
             broker = get_broker()
             context.update(
-                broker.processor_backend.get_payment_context(
-                    broker,
-                    self.organization.processor_card_key,
-                    subscriber_email=self.organization.email,
-                    subscriber_slug=self.organization.slug))
+                broker.processor_backend.get_payment_context(# card update
+                    self.organization,
+                    provider=broker, broker=broker))
         except ProcessorConnectionError:
             messages.error(self.request, _("The payment processor is "\
                 "currently unreachable. Sorry for the inconvienience."))
@@ -446,6 +443,13 @@ class AllTransactions(ProviderMixin, TransactionBaseView):
 
     template_name = 'saas/billing/transactions.html'
 
+    def get_context_data(self, **kwargs):
+        context = super(AllTransactions, self).get_context_data(**kwargs)
+        update_context_urls(context, {
+            'download': reverse('saas_transactions_download',
+                kwargs=self.get_url_kwargs(**kwargs))
+        })
+        return context
 
 class BillingStatementView(OrganizationMixin, TransactionBaseView):
     """
@@ -458,7 +462,8 @@ class BillingStatementView(OrganizationMixin, TransactionBaseView):
     ``saas/billing/index.html`` (`example <https://github.com/djaodjin\
 /djaodjin-saas/tree/master/saas/templates/saas/billing/index.html>`__).
     You should insure the page will call back the
-    :ref:`/api/billing/:organization/payments/ <api_billing_payments>`
+    `/api/billing/{profile}/history <https://www.djaodjin.com/docs/\
+reference/djaoapp/latest/api/#listBillings>`__
     API end point to fetch the set of transactions.
 
     Template context:
@@ -472,12 +477,11 @@ class BillingStatementView(OrganizationMixin, TransactionBaseView):
 
     def get_context_data(self, **kwargs):
         context = super(BillingStatementView, self).get_context_data(**kwargs)
-        context.update({
-            'organization': self.organization,
-            'download_url': reverse('saas_statement_download',
-                kwargs=self.get_url_kwargs(**kwargs))})
-        update_context_urls(context,
-            {'organization': {
+        context.update({'organization': self.organization})
+        update_context_urls(context, {
+            'download': reverse('saas_statement_download',
+                kwargs=self.get_url_kwargs(**kwargs)),
+            'organization': {
                 'api_transactions': reverse(
                     'saas_api_billings', args=(self.organization,)),
                 'balance': reverse(
@@ -503,8 +507,9 @@ class TransferListView(BankMixin, TransactionBaseView):
     To edit the layout of this page, create a local \
     ``saas/billing/transfers.html`` (`example <https://github.com/djaodjin/\
 djaodjin-saas/tree/master/saas/templates/saas/billing/transfers.html>`__).
-    You should insure the page will call back the
-    :ref:`/api/billing/:organization/transfers/ <api_billing_transfers>`
+    You should insure the page will call the
+    `/api/billing/{profile}/transfers <https://www.djaodjin.com/docs/\
+reference/djaoapp/latest/api/#listTransfer>`__
     API end point to fetch the set of transactions.
 
     Template context:
@@ -515,10 +520,9 @@ djaodjin-saas/tree/master/saas/templates/saas/billing/transfers.html>`__).
 
     def get_context_data(self, **kwargs):
         context = super(TransferListView, self).get_context_data(**kwargs)
-        context.update({
-            'download_url': reverse('saas_transfers_download',
-                kwargs=self.get_url_kwargs(**kwargs))})
-        urls = {
+        update_context_urls(context, {
+            'download': reverse('saas_transfers_download',
+                kwargs=self.get_url_kwargs(**kwargs)),
             'organization': {
                 'api_transactions': reverse(
                 'saas_api_transfer_list', args=(self.provider,)),
@@ -529,8 +533,13 @@ djaodjin-saas/tree/master/saas/templates/saas/billing/transfers.html>`__).
                     'saas_import_transactions', args=(self.provider,)),
                 'withdraw_funds': reverse(
                     'saas_withdraw_funds', args=(self.provider,)),
-        }}
-        update_context_urls(context, urls)
+        }})
+        if self.provider.is_broker:
+            update_context_urls(context, {
+                'raw_charges': reverse('saas_charges'),
+                'raw_transactions': reverse('saas_broker_transactions')
+            })
+
         return context
 
 
@@ -869,10 +878,11 @@ class ChargeListView(ProviderMixin, TemplateView):
 
     def get_context_data(self, **kwargs):
         context = super(ChargeListView, self).get_context_data(**kwargs)
-        urls = {'broker': {
-            'api_charges': reverse('saas_api_charges'),
-        }}
-        update_context_urls(context, urls)
+        update_context_urls(context, {
+            'download': reverse('saas_charges_download'),
+            'broker': {
+                'api_charges': reverse('saas_api_charges'),
+            }})
         return context
 
 
@@ -940,8 +950,9 @@ class CouponListView(DateRangeContextMixin, ProviderMixin, TemplateView):
     ``saas/billing/coupons.html`` (`example <https://github.com/\
 djaodjin/djaodjin-saas/tree/master/saas/templates/saas/billing/\
 coupons.html>`__).
-    You should insure the page will call back the
-    :ref:`/api/billing/:organization/coupons/ <api_billing_coupons>`
+    You should insure the page will call the
+    `/api/billing/{profile}/coupons <https://www.djaodjin.com/docs/\
+reference/djaoapp/latest/api/#listCouponListCreate>`
     API end point to fetch the set of coupons for the provider organization.
 
 
